﻿using FCS.Models;
using FCS.Models.Entity;
using System.Collections.Generic;

namespace FCS.Interface
{
    /// <summary>
    /// 地方彩相关数据服务接口
    /// </summary>
    public interface IDTOpenCode
    {
        /// <summary>
        /// 存取国家信息
        /// </summary>
        /// <param name="lottery"></param>
        /// <param name="model"></param>
        /// <returns></returns>
        bool AddArea(FCSLottery lottery, F_Area model);

        /// <summary>
        /// 赛事添加
        /// </summary>
        /// <param name="lottery"></param>
        /// <param name="model"></param>
        /// <returns></returns>
        bool AddEvents(FCSLottery lottery, F_Events model);

        /// <summary>
        /// 获取枚举值
        /// </summary>
        /// <returns></returns>
        List<DataItemDetail> GetDataItem(DataItemDetailEnum dateItem);
        /// <summary>
        /// 添加球队信息
        /// </summary>
        /// <param name="lottery"></param>
        /// <param name="model"></param>
        /// <returns></returns>
        bool AddTeam(FCSLottery lottery, F_Team model);

        /// <summary>
        /// 添加球队信息
        /// </summary>
        /// <param name="lottery"></param>
        /// <param name="model"></param>
        /// <returns></returns>
        bool AddPlayers(FCSLottery lottery, F_Players model);

        /// <summary>
        /// 添加球队球员信息
        /// </summary>
        /// <param name="lottery"></param>
        /// <param name="model"></param>
        /// <returns></returns>
        bool AddRelation(FCSLottery lottery, F_Relation model);

        /// <summary>
        /// 根据国家名字查错Guid
        /// </summary>
        /// <param name="lottery"></param>
        /// <param name="model"></param>
        /// <returns></returns>
        F_EuropeanEvents GetAreaId(string Name);
        /// <summary>
        /// 批量新增（SqlBulkCopy）
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="list"></param>
        void SqlBulkCopyAdd<T>(IEnumerable<T> list);
        /// <summary>
        /// 获取球队列表
        /// </summary>
        /// <returns></returns>
        List<F_Team> GetTeamList();
        /// <summary>
        /// 获取球员列表
        /// </summary>
        /// <returns></returns>
        List<F_Players> GetPlayerList();

        /// <summary>
        /// 获取国家列表
        /// </summary>
        /// <returns></returns>
        List<F_Area> GetContryList();

        /// <summary>
        /// 单表查询
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <returns></returns>
        IEnumerable<T> Query<T>(string where = "", string json = "", string select = "a.*");
        /// <summary>
        /// 表查询
        /// </summary>
        /// <typeparam name="T">表名对象</typeparam>
        /// <typeparam name="U">转换类型</typeparam>
        /// <param name="where">条件</param>
        /// <param name="json">join条件（默认第一个是a开头）</param>
        /// <param name="select">查询字段</param>
        /// <returns></returns>
         IEnumerable<U> Query<T, U>(string where = "", string json = "", string select = "a.*");
        /// <summary>
        /// 单表查询
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <returns></returns>
        IEnumerable<T> Query1<T>(string where = "");
        void Delete<T>(string where = "");

        void Update<T>(string where = "");
        #region 新闻相关
        /// <summary>
        /// 新闻添加
        /// </summary>
        /// <param name="lottery"></param>  
        /// <param name="model"></param>
        /// <returns></returns>
        bool AddNews(NewsTypeEnum news, Base_News model);

        #endregion

        /// <summary>
        /// 获取篮球球队
        /// </summary>
        /// <returns></returns>
        List<B_Team> GetBasketTeamList();
        
        /// <summary>
        /// 获取篮球球员
        /// </summary>
        /// <returns></returns>
        List<B_Players> GetBasketPalyerList();
    }
}